% ================ Programma Analisi_Coppia.m ================= %
%                                                               %
%              ESERCITAZIONE di ANALISI NUMERICA                %
% 								                                %
%  Applicazione della FFT all'analisi armonica della coppia di  %
%  un motore a combustione interna		                      	%
% 								                                %
% ============================================================= %

% Il programma richiede i programmi esterni:
%
%		- ang.mat    (angoli di manovella)
%       - data.mat   (valori di pressione in camera di comb.)
%		

clear
close all

% Caricamento dei dati di coppia di motore monocilindrico 4T
load ang
load data
datamax = max(data);
datamean = mean(data);

% Velocit di rotazione
omega = input('Velocit di rotazione in giri/min = ');
omega = 2*pi*omega/60;
periodo = 4*pi/omega

pause

ang = ang/720*periodo;                       % Angolo temporale

% Disegno della coppia del motore monocilindrico
figure(1)
plot(ang,data,'b','LineWidth',3)
axis([0 periodo -200 800])
text(ang(1000),620,['Coppia media = ',num2str(datamean),' N*m'])
title('Coppia motrice MCI monocilindrico')
xlabel('Periodo temporale (2 giri) - (s)')
ylabel('Coppia istantanea (Nm)')
grid on
hold on
plot([0 720],[datamean datamean],'r')
hold off

pause

% Calcolo e disegno della coppia di un motore quadricilindrico
dataa=data(1:4096)+data(4097:8192)+data(8193:12288)+data(12289:16384);
data4 = [dataa dataa dataa dataa dataa(1)];
datamax4 = max(data4);
datamean4 = mean(data4);

figure(2)
plot(ang,data4,'b','LineWidth',3)
axis([0 periodo -200 800])
text(ang(1000),620,['Coppia media = ',num2str(datamean4),' N*m'])
title('Coppia motrice MCI quadricilindrico')
xlabel('Periodo temporale (2 giri) - (s)')
ylabel('Coppia istantanea (Nm)')
grid on
hold on
plot([0 720],[datamean4 datamean4],'r')
hold off

pause

% Analisi del risultato della fft in termini di ampiezza e fase
% per il motore monocilindrico 
% Analisi dell'influenza dei punti campionati (8    16    32    64   128
% 256   512)

for power = 3:9

n = 2^power;

% Motore monocilindrico

mystep = 16384/n;

dt = periodo/n;
freq = 1/dt;
angf = zeros(1,n+1);
dataf = zeros(1,n+1);

a = 1:n; a= [0 a];
f = freq/n*a;

for i=1:mystep:16385
    ii = 1+(i-1)/mystep;
    angf(ii) = ang(i);
    dataf(ii)= data(i);
end

Y = fft(dataf,n);
A = 2*abs(Y)/n;
A(1) = Y(1)/n;
F = angle(Y);

figure(3)
  
    limit = n/2;
    
	subplot(2,1,1)    
	stem(f(1:limit),A(1:limit),'b')
	axis([-1 f(limit)+1  0 1.5*max(A)])
   	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Ampiezza')
	title('Analisi armonica onda quadra (1 cilindro)')
    hold on
    for ii=1:512
        plot([1/periodo*ii 1/periodo*ii],[0 1.5*max(A)],'m')
    end
	stem(f(1:limit),A(1:limit),'b')
    hold off
    
	subplot(2,1,2)
   	stem(f(1:limit),F(1:limit)*180/pi,'b')
	axis([-1 f(limit)+1  -180 180])
	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Fase')
    hold on
    plot([0 f(limit)],[0 0],'r')
    for ii=1:512
        plot([1/periodo*ii 1/periodo*ii],[-180 180],'m')
    end
   	stem(f(1:limit),F(1:limit)*180/pi,'b')
    hold off
		
pause

datarec = ifft(Y);
datarec = [datarec datarec(1)];
datarec = real(datarec);
datarec1 = interp1(angf,datarec,ang,'spline');
err = abs(datarec1-data)/datamax*100;

figure(4)

h1 = subplot(3,1,1);
  plot(ang,data,':b')
  axis([0 periodo -200 800])
  ylabel('Campioni (Nm)')
  grid on
  title(['Numero campioni = ', num2str(n), '; \Delta t = ',num2str(dt),' s; fc = ',num2str(freq),' Hz'])
  hold on
  stem(angf,dataf,'r')
  hold off

h2 = subplot(3,1,2);
  plot(ang,data,'m','LineWidth',3) 
  axis([0 periodo -200 800])
  ylabel('Coppia (Nm)')
  grid on
  hold on
  plot(ang,datarec1,'k')
  media = mean(datarec1);
  text(ang(500),620,['Coppia media = ',num2str(media),' N*m'])
  text(ang(500),250,['Coppia media vera = ',num2str(datamean),' N*m'])
  hold off
  
h3 = subplot(3,1,3);
  plot(ang,err,'r')
  axis([0 periodo 0 max(err)])
  xlabel('Tempo [s]')
  ylabel('Errore (%)')
  grid on
  
pause

end

    
% =============================================================== %
%							                                  	  %
%                    Termine del programma    	         		  %
%								                                  % 
% =============================================================== %