% ================ Programma fft_onda_quadra.m ================ %
%                                                               %
%              ESERCITAZIONE di ANALISI NUMERICA                %
% 								                                %
% Applicazione della FFT all'analisi armonica di un'onda quadra %
% 							                                	%
% ============================================================= %

% Il programma richiede i programmi esterni:
%
%		ondaq.m
%		

clear all
close all

% Costruzione di un'onda quadra di riferimento

	x1 = 0; xq = 1.5; x2 = 3; ver = 0;
    
    timestep = 0.01;   % Intervallo temporale di rappresentazione
	x = 0:timestep:8;
    
	amp = 2;            % Ampiezza onda quadra
    tper = x2;          % Periodo dell'onda quadra
    
	yq = ondaq(x,amp/2,x1,tper,xq,ver,max(x),timestep);

	figure(1)

	plot(x,yq,'b','LineWidth',3) 
    minx = min(x);
    maxx = max(x);
	axis([minx maxx min(yq)-0.2 max(yq)+0.2])
	grid on
	xlabel('Tempo [s]')
	ylabel('Ampiezza')
    hold on
    plot([3 3],[-2 4],'m',[0 max(x)],[0 0],'m')
    hold off
    
disp('Per eseguire la FFT digitare "ENTER"')
pause
    
% ============================================================

% Esecuzione dell'analisi armonica mediante FFT ed analisi del 
% risultato ottenuto al variare dei punti di campionamento

% Definizione del campionamento

    ncamp = input('Numero dei punti campionati = ')
    
	tper = x2;                         % periodo
    dt =tper/ncamp;                    % step nei tempi
	fonda = 1/tper;                    % frequenza fondamentale
	fcamp = ncamp/tper;                % freq. di campionamento
	f = fonda*(0:(ncamp/2-1));         % contenuto in frequenze

% Campionamento della funzione    
	x = 0:dt:x2-dt;                    % Vettore di ncamp valori
	yq = ondaq(x,amp/2,x1,tper,xq,ver,max(x),dt);
    
% Uso della FFT

	Y = fft(yq,ncamp);
    
    A = 2*abs(Y(1:ncamp/2))/ncamp;
	F = angle(Y(1:ncamp/2));
	A(1) = Y(1)/ncamp;
    F(1) = 0;

    
% Analisi del risultato della fft in termini di parte reale ed immaginaria

    figure(2)
    f1 = 1:1:ncamp;
    limite = max([abs(min(real(Y))) abs(max(real(Y)))]);
    subplot(2,1,1)
	bar(f1,real(Y(1:ncamp)),'b')
	axis([1 ncamp -limite limite])
	grid on
	ylabel('Real')
	title(['Analisi armonica onda quadra; ncamp = ',num2str(ncamp),...
            '; valor medio = ', num2str(Y(1)/ncamp),...
            '; Y(1) = ',num2str(Y(1))])
    hold on
    plot([1 ncamp],[0 0],'r')
    for i=1:ncamp
        if real(Y(i)) == 0
            plot(f1(i),real(Y(i)),'or')
        end
    end
    hold off
    
    subplot(2,1,2)
	bar(f1,imag(Y(1:ncamp)),'b')
	axis([1 ncamp min(imag(Y)) max(imag(Y))])
	grid on
	xlabel(['Indice k (1-',num2str(ncamp),')'])
	ylabel('Imag')
    hold on
    plot([1 ncamp],[0 0],'r')
    for i=1:ncamp
        if imag(Y(i)) == 0
            plot(f1(i),imag(Y(i)),'or')
        end
    end
    hold off
    
    disp('Per analizzare il risultato in ampiezza e fase digitare "ENTER"')
	pause
    
% Analisi del risultato della fft in termini di ampiezza e fase

   	figure(3)
    
    limit = ncamp/2;
    
	subplot(2,1,1)
	bar(f(1:limit),A(1:limit))
	axis([0 f(limit)  0 1.5*max(A)])
   	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Ampiezza')
	title('Analisi armonica onda quadra')
    
	subplot(2,1,2)
	bar(f(1:limit),F(1:limit)*180/pi,'b')
	axis([0 f(limit)  -180 180])
	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Fase')
    hold on
    plot([0 f(limit)],[0 0],'r')
    hold off
    
disp('Per analizzare l''uso della trasformata inversa digitare "ENTER"')
pause

% Ricostruzione della funzione mediante la fft inversa

% Costruzione dell'onda quadra di riferimento  e disegno
    xd = 0:timestep:x2-timestep;
    yqd = ondaq(xd,amp/2,x1,tper,xq,ver,max(x),timestep); 

    % Costruzione del campionamento dell'onda quadra
    % ncamp1=32;
    % dt = x2/ncamp1

   	xx = 0:dt:x2-dt;
	yq1 = ondaq(xx,amp/2,x1,tper,xq,ver,max(x),dt); 
    
    % Calcolo FFT e FFT inversa
	Y1 = fft(yq1,ncamp);                  % Calcolo della FFT  
	
    IY = ifft(Y1,ncamp);                  % Ricostruzione tramite IFFT  
    
    % Plottaggio del risultato    
    figure(4)
    plot([0 0],[min(yq)-0.2 max(yq)+0.2],'m',[3 3],...
        [min(yq)-0.2 max(yq)+0.2],'m')   
    axis([-0.1 3.1 min(yq)-0.2 max(yq)+0.2])
	grid on
	xlabel('Tempo [s]')
	ylabel('Ampiezza')
	title('Ricostruzione onda quadra mediante FFT inversa')
    
   	hold on
    plot(xd,yqd,'c','LineWidth',3)
    text(1.6,0.5,'Onda Q di riferimento')
    
    pause
    plot(xx,yq1,'ob',[-1 4],[0 0],'m')
    text(1.6,0.3,'o= Punti campionati')

    pause
	plot(xx(1:ncamp),real(IY),'*k')
    text(1.6,0.1,'* = Punti ricostruiti con IFFT')
	hold off

disp('Per ricostruire come somma di funzioni armoniche digitare "ENTER"')
pause	

% Ricostruzione della funzione mediante somma delle funzioni armoniche
%                    A*cos(2*pi*k*t/tper + fase) 

   figure(5)
    
    y = A(1);
    tt = 0:tper/180:tper;
    for j = 2:ncamp/2
        k = j-1;
        y = y + A(j)*cos(2*pi*k*tt/tper + F(j)); 
        plot(tt,y,'b','LineWidth',3)
        axis([0 tper min(yq)-0.4 max(yq)+0.4])
        grid on
        title(['Ricostruzione con armoniche ',num2str(j-1)])
        xlabel('Tempo [s]')
        ylabel('Ampiezza')
        pause(0.5)
        if j < ncamp/2
            plot(tt,y,'w')
        end
    end
    
    
% =============================================================== %
%							                                  	  %
%                    Termine del programma    	         		  %
%								                                  % 
% =============================================================== %