% =============== Programma fft_onda_quadra2.m ================ %
%                                                               %
%              ESERCITAZIONE di ANALISI NUMERICA                %
% 								                                %
% Applicazione della FFT all'analisi armonica di un'onda quadra %
% con possibilit di variazione dei parametri principali       	%
% ============================================================= %

% Il programma richiede i programmi esterni:
%
%		- ondaq.m
%		

clear all
close all

% Costruzione di un'onda quadra di riferimento

	x1 = 0; xq = 1.5; x2 = 3; ver = 0;
    
    tper = x2;
	fonda = 1/tper;
    ncamp=128;
    dt = tper/ncamp;
    
  	x = 0:dt:tper-dt;
	amp = 2;
	yq = ondaq(x,amp/2,x1,tper,xq,ver,tper,dt);

	figure(1)

	plot(x,yq,'b','LineWidth',3)
    minx = min(x);
    maxx = max(x);
	axis([minx maxx -2 4])
	grid on
	xlabel('Tempo [s]')
	ylabel('Ampiezza')
    hold on
    plot([3 3],[-2 4],'m',[0 max(x)],[0 0],'m')
    hold off
    
    pause
    
% Sliders per cambiare i parametri dell'onda quadra

% Slider per modificare il punto di discesa intermedia

	all_cur = xq;
	sli_all = uicontrol(gcf,...
		'Style','slider',...
		'Position',[(10+250)  (10+355)  200  24],...
		'Min',0.1,'Max',2.9,'Value',xq,...
		'CallBack',['all_cur = get(sli_all,''Val'');',...
		'xq = all_cur;',...
		'yq = ondaq(x,amp/2,x1,tper,xq,ver,tper,dt);,',...
		'plot(x,yq,''b'',''LineWidth'',3);hold on;plot([3 3],[-2 4],''m'',[0 max(x)],[0 0],''m'');hold off;',...
		'axis([minx maxx -2 4]);,',...
		'grid;,',...
		'xlabel(''Tempo [s]'');,',...
		'ylabel(''Ampiezza'');,',...
		'set(allu_cur,''String'',num2str(all_cur));']);
	all_label = uicontrol(gcf,...
		'Style','text',...
		'String','Punto discesa',...
		'Position',[(65+250) (40+355) 90 20]);
	all_min = uicontrol(gcf,...
		'Style','text',...
		'Position',[(10+250) (40+355) 50 20],...
		'String',num2str(get(sli_all,'Min')));
	all_max = uicontrol(gcf,...
		'Style','text',...
		'Position',[(160+250) (40+355) 50 20],...
		'String',num2str(get(sli_all,'Max')));
	allu_cur = uicontrol(gcf,...
		'Style','text',...
		'Pos',[470 (10+355) 50 50],...
		'String',num2str(get(sli_all,'Value')));


% Slider per modificare lo shift orizzontale

	sho_cur = x1;
	sli_sho = uicontrol(gcf,...
		'Style','slider',...
		'Position',[(10+250)  (10+295)  200  24],...
		'Min',-1.5,'Max',1.5,'Value',x1,...
		'CallBack',['sho_cur = get(sli_sho,''Val'');',...
		'x1 = sho_cur;',...
		'yq = ondaq(x,amp/2,x1,tper,xq,ver,tper,dt);,',...
		'plot(x,yq,''b'',''LineWidth'',3);hold on;plot([3 3],[-2 4],''m'',[0 max(x)],[0 0],''m'');hold off;',...
		'axis([minx maxx -2 4]);,',...
		'grid;,',...
		'xlabel(''Tempo [s]'');,',...
		'ylabel(''Ampiezza'');,',...
		'set(shou_cur,''String'',num2str(sho_cur));'] );
	sho_label = uicontrol(gcf,...
		'Style','text',...
		'String','Shift orizz.',...
		'Position',[(65+250) (40+295) 90 20]);
	sho_min = uicontrol(gcf,...
		'Style','text',...
		'Position',[(10+250) (40+295) 50 20],...
		'String',num2str(get(sli_sho,'Min')));
	sho_max = uicontrol(gcf,...
		'Style','text',...
		'Position',[(160+250) (40+295) 50 20],...
		'String',num2str(get(sli_sho,'Max')));
	shou_cur = uicontrol(gcf,...
		'Style','text',...
		'Pos',[470 (10+295) 50 50],...
		'String',num2str(get(sli_sho,'Value')));

% Slider per modificare lo shift verticale

	shv_cur = ver;
	sli_shv = uicontrol(gcf,...
		'Style','slider',...
		'Position',[(10+250)  (10+240)  200  24],...
		'Min',-1,'Max',1,'Value',ver,...
		'CallBack',['shv_cur = get(sli_shv,''Val'');',...
		'ver = shv_cur;',...
		'yq = ondaq(x,amp/2,x1,tper,xq,ver,tper,dt);,',...
		'plot(x,yq,''b'',''LineWidth'',3);hold on;plot([3 3],[-2 4],''m'',[0 max(x)],[0 0],''m'');hold off;',...
		'axis([minx maxx -2 4]);,',...
		'grid;,',...
		'xlabel(''Tempo [s]'');,',...
		'ylabel(''Ampiezza'');,',...
		'set(shvu_cur,''String'',num2str(shv_cur));'] );
	shv_label = uicontrol(gcf,...
		'Style','text',...
		'String','Shift verticale',...
		'Position',[(65+250) (40+240) 90 20]);
	shv_min = uicontrol(gcf,...
		'Style','text',...
		'Position',[(10+250) (40+240) 50 20],...
		'String',num2str(get(sli_shv,'Min')));
	shv_max = uicontrol(gcf,...
		'Style','text',...
		'Position',[(160+250) (40+240) 50 20],...
		'String',num2str(get(sli_shv,'Max')));
	shvu_cur = uicontrol(gcf,...
		'Style','text',...
		'Pos',[470 (10+240) 50 50],...
		'String',num2str(get(sli_shv,'Value')));

	pause
	
	x = 0:dt:x2-dt;
	yq = ondaq(x,amp/2,x1,tper,xq,ver,tper,dt);
	Y = fft(yq,ncamp);
    
	A = 2*abs(Y(1:ncamp/2))/ncamp;
	F = angle(Y(1:ncamp/2));
	A(1) = Y(1)/ncamp;
    F(1) = 0;
	fcamp = ncamp/tper;
	f = (fcamp/ncamp)*(0:(ncamp/2-1));

% Analisi del risultato della fft in termini di ampiezza e fase

   	figure(2)
    
    f1 = 0:1:ncamp-1;

    subplot(2,1,1)
	bar(f1,real(Y(1:ncamp)),'b')
	axis([-1 ncamp min(real(Y)) max(real(Y))])
	grid on
	ylabel('Real')
	title(['Analisi armonica onda quadra; ncamp = ',num2str(ncamp),...
            '; valor medio = ', num2str(Y(1)/ncamp),...
            '; Y(1) = ',num2str(Y(1))])
 
    hold on
    plot([0 ncamp-1],[0 0],'r')
    hold off
    
    subplot(2,1,2)
	bar(f1,imag(Y(1:ncamp)),'b')
	axis([-1 ncamp min(imag(Y)) max(imag(Y))])
	grid on
	xlabel(['Indice k (0-',num2str(ncamp-1),')'])
	ylabel('Imag')
    hold on
    plot([0 ncamp-1],[0 0],'r')
    hold off

    pause
    
    figure(3)
    
    limit = ncamp/2;
    
	subplot(2,1,1)
	bar(f(1:limit),A(1:limit))
	axis([-1 f(limit)+1  0 1.5*max(A)])
   	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Ampiezza')
	title('Analisi armonica onda quadra')
	
	subplot(2,1,2)
	bar(f(1:limit),F(1:limit)*180/pi,'b')
	axis([-1 f(limit)+1  -180 180])
	grid on
	xlabel('Frequenza (Hz)')
	ylabel('Fase')
    hold on
    plot([0 f(limit)],[0 0],'r')
    hold off
    
    pause
		
% Ricostruzione della funzione mediante somma delle funzioni armoniche
%                    A*cos(2*pi*k*t/tper + fase) 

   figure(4)
    
    y = A(1);
    tt = 0:tper/180:tper;
    miny = 1.4*(-amp+ver); 
    maxy = 1.4*(amp+ver);
       
    for j = 2:ncamp/2
        k = j-1;
        y = y + A(j)*cos(2*pi*k*tt/tper + F(j)); 
        plot(tt,y,'b','LineWidth',3)
        axis([0 tper -2 4])
        grid on
        title(['Ricostruzione con armoniche ',num2str(j-1)])
        xlabel('Tempo [s]')
        ylabel('Ampiezza')
        pause(0.5)
        if j < ncamp/2
            plot(tt,y,'w')
        end
    end
    
% =============================================================== %
%							                                  	  %
%                    Termine del programma    	         		  %
%								                                  % 
% =============================================================== %